@@ -18,23 +18,13 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-bin.h,v 1.2 2001/09/12 13:42:15 ela Exp $
+ * $Id: guile-bin.h,v 1.4 2001/11/19 13:31:50 ela Exp $
  *
  */
 
 #ifndef __GUILE_BIN_H__
 #define __GUILE_BIN_H__ 1
 
-/* Some definitions for backward compatibility with Guile 1.3.4 */
-#ifndef SCM_ASSERT_TYPE
-# define SCM_ASSERT_TYPE(_cond, _arg, _pos, _subr, _msg) \
-    SCM_ASSERT (_cond, _arg, _pos, _subr)
-#define scm_wrong_type_arg_msg(_subr, _pos, _bad, _msg) \
-    scm_wrong_type_arg (_subr, _pos, _bad)
-#define scm_out_of_range_pos(_subr, _bad, _pos) \
-    scm_out_of_range (_subr, _bad)
-#endif /* ! SCM_ASSERT_TYPE */
-
 void guile_bin_init (void);
 int guile_bin_check (SCM);
 SCM guile_data_to_bin (void *, int);
