@@ -1,5 +1,5 @@
 /*
- * guile.c - interface to guile core library
+ * guile.c - interface to Guile core library
  *
  * Copyright (C) 2001 Raimund Jacob <raimi@lkcc.org>
  * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile.c,v 1.44 2001/09/21 13:12:41 ela Exp $
+ * $Id: guile.c,v 1.53 2001/11/29 23:41:43 raimi Exp $
  *
  */
 
@@ -39,6 +39,7 @@
 #endif
 
 #include "libserveez.h"
+#include "guile-api.h"
 #include "guile-server.h"
 #include "guile.h"
 
@@ -131,7 +132,7 @@ static SCM
 guile_get_current_load_port (void)
 {
   SCM p = scm_current_load_port ();
-  if (!gh_eq_p (p, SCM_BOOL_F) && SCM_PORTP (p))
+  if (!SCM_FALSEP (p) && SCM_PORTP (p))
     return p;
   if (SCM_PORTP (guile_load_port))
     return guile_load_port;
@@ -157,7 +158,7 @@ guile_error (char *format, ...)
 	   SCM_PORTP (lp) ? SCM_LINUM (lp) + 1 : 0, 
 	   SCM_PORTP (lp) ? SCM_COL (lp) : 0);
   if (file)
-    scm_must_free (file);
+    scm_c_free (file);
 
   va_start (args, format);
   vfprintf (stderr, format, args);
@@ -246,24 +247,24 @@ guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
   int err = 0;
 
   /* Unpack if requested, ignore if null already (null == not existent). */
-  if (dounpack && !gh_null_p (pairlist) && !gh_eq_p (pairlist, SCM_UNDEFINED))
-    pairlist = gh_car (pairlist);
+  if (dounpack && !SCM_NULLP (pairlist) && !SCM_UNBNDP (pairlist))
+    pairlist = SCM_CAR (pairlist);
 
-  for ( ; gh_pair_p (pairlist); pairlist = gh_cdr (pairlist))
+  for ( ; SCM_PAIRP (pairlist); pairlist = SCM_CDR (pairlist))
     {
-      SCM pair = gh_car (pairlist);
+      SCM pair = SCM_CAR (pairlist);
       SCM key, val;
       char *str = NULL;
 
       /* The car must be another pair which contains key and value. */
-      if (!gh_pair_p (pair))
+      if (!SCM_PAIRP (pair))
 	{
 	  guile_error ("Not a pair %s", txt);
 	  err = 1;
 	  break;
 	}
-      key = gh_car (pair);
-      val = gh_cdr (pair);
+      key = SCM_CAR (pair);
+      val = SCM_CDR (pair);
 
       if (NULL == (str = guile_to_string (key)))
 	{
@@ -282,11 +283,11 @@ guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
 	  svz_free_and_zero (old_value);
 	}
       svz_hash_put (hash, str, (void *) new_value);
-      scm_must_free (str);
+      scm_c_free (str);
     }
 
-  /* Pairlist must be gh_null_p() now or that was not a good pairlist. */
-  if (!err && !gh_null_p (pairlist))
+  /* Pairlist must be SCM_NULLP() now or that was not a good pairlist. */
+  if (!err && !SCM_NULLP (pairlist))
     {
       guile_error ("Invalid pairlist %s", txt);
       err = 1;
@@ -313,9 +314,9 @@ guile_to_integer (SCM cell, int *target)
   char *str = NULL, *endp;
 
   /* Usual guile exact number. */
-  if (gh_exact_p (cell))
+  if (SCM_EXACTP (cell))
     {
-      *target = gh_scm2int (cell);
+      *target = SCM_NUM2INT (SCM_ARG1, cell);
     }
   /* Try string (or even symbol) to integer conversion. */
   else if (NULL != (str = guile_to_string (cell)))
@@ -324,7 +325,7 @@ guile_to_integer (SCM cell, int *target)
       *target = strtol (str, &endp, 10);
       if (*endp != '\0' || errno == ERANGE)
 	err = 1;
-      scm_must_free (str);
+      scm_c_free (str);
     }
   /* No chance. */
   else
@@ -348,9 +349,9 @@ guile_to_boolean (SCM cell, int *target)
   char *str;
 
   /* Usual guile boolean. */
-  if (gh_boolean_p (cell))
+  if (SCM_BOOLP (cell))
     {
-      i = gh_scm2bool (cell);
+      i = SCM_NFALSEP (cell);
       *target = (i == 0 ? 0 : 1);
     }
   /* Try with the integer converter. */
@@ -371,6 +372,7 @@ guile_to_boolean (SCM cell, int *target)
 	*target = 0;
       else
 	err = 1;
+      scm_c_free (str);
     }
   else
     {
@@ -383,6 +385,7 @@ guile_to_boolean (SCM cell, int *target)
  * Convert the given guile list @var{list} into a hash. Return NULL on
  * failure. Error messages will be emitted if necessary.
  */
+#define FUNC_NAME "guile_to_hash"
 svz_hash_t *
 guile_to_hash (SCM list, char *prefix)
 {
@@ -390,7 +393,7 @@ guile_to_hash (SCM list, char *prefix)
   svz_hash_t *hash;
 
   /* Is this valid guile list at all ? */
-  if (!gh_list_p (list))
+  if (!SCM_LISTP (list))
     {
       err = -1;
       guile_error ("%s: Not a valid list for hash (alist)", prefix);
@@ -398,21 +401,21 @@ guile_to_hash (SCM list, char *prefix)
     }
 
   /* Iterate the alist. */
-  hash = svz_hash_create (gh_length (list));
-  for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
+  hash = svz_hash_create (SCM_NUM2ULONG (SCM_ARG1, scm_length (list)));
+  for (i = 0; SCM_PAIRP (list); list = SCM_CDR (list), i++)
     {
-      SCM k, v, pair = gh_car (list);
+      SCM k, v, pair = SCM_CAR (list);
       char *str, *keystr, *valstr;
 
-      if (!gh_pair_p (pair))
+      if (!SCM_PAIRP (pair))
 	{
 	  err = -1;
 	  guile_error ("%s: Element #%d of hash (alist) is not a pair", 
 		       prefix, i);
 	  continue;
 	}
-      k = gh_car (pair);
-      v = gh_cdr (pair);
+      k = SCM_CAR (pair);
+      v = SCM_CDR (pair);
 	     
       /* Obtain key character string. */
       if (NULL == (str = guile_to_string (k)))
@@ -425,7 +428,7 @@ guile_to_hash (SCM list, char *prefix)
       else
 	{
 	  keystr = svz_strdup (str);
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
 
       /* Obtain value character string. */
@@ -439,7 +442,7 @@ guile_to_hash (SCM list, char *prefix)
       else
 	{
 	  valstr = svz_strdup (str);
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
 
       /* Add to hash if key and value look good. */
@@ -461,6 +464,7 @@ guile_to_hash (SCM list, char *prefix)
     }
   return hash;
 }
+#undef FUNC_NAME
 
 /*
  * Convert the given scheme cell @var{list} which needs to be a valid guile
@@ -469,6 +473,7 @@ guile_to_hash (SCM list, char *prefix)
  * is not a string. The additional argument @var{func} should be the name of 
  * the calling function.
  */
+#define FUNC_NAME "guile_to_strarray"
 svz_array_t *
 guile_to_strarray (SCM list, char *func)
 {
@@ -477,24 +482,25 @@ guile_to_strarray (SCM list, char *func)
   char *str;
 
   /* Check if the given scheme cell is a valid list. */
-  if (!gh_list_p (list))
+  if (!SCM_LISTP (list))
     {
       guile_error ("%s: String array is not a valid list", func);
       return NULL;
     }
 
   /* Iterate over the list and build up the array of strings. */
-  array = svz_array_create (gh_length (list));
-  for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
+  array = svz_array_create (SCM_NUM2ULONG (SCM_ARG1, scm_length (list)), 
+			    svz_free);
+  for (i = 0; SCM_PAIRP (list); list = SCM_CDR (list), i++)
     {
-      if ((str = guile_to_string (gh_car (list))) == NULL)
+      if ((str = guile_to_string (SCM_CAR (list))) == NULL)
 	{
 	  guile_error ("%s: String expected in position %d", func, i);
 	  guile_global_error = -1;
 	  continue;
 	}
       svz_array_add (array, svz_strdup (str));
-      scm_must_free (str);
+      scm_c_free (str);
     }
 
   /* Check the size of the resulting string array. */
@@ -505,12 +511,14 @@ guile_to_strarray (SCM list, char *func)
     }
   return array;
 }
+#undef FUNC_NAME
 
 /*
  * Convert the given scheme cell @var{list} which needs to be a valid guile
  * list into an array of integers. The additional argument @var{func} is the
  * name of the calling function. Return NULL on failure.
  */
+#define FUNC_NAME "guile_to_intarray"
 svz_array_t *
 guile_to_intarray (SCM list, char *func)
 {
@@ -518,17 +526,17 @@ guile_to_intarray (SCM list, char *func)
   int i, n;
 
   /* Check if the given scheme cell is a valid associative list. */
-  if (!gh_list_p (list))
+  if (!SCM_LISTP (list))
     {
       guile_error ("%s: Integer array is not a valid list", func);
       return NULL;
     }
 
   /* Iterate over the list and build up the array of strings. */
-  array = svz_array_create (gh_length (list));
-  for (i = 0; gh_pair_p (list); list = gh_cdr (list), i++)
+  array = svz_array_create (SCM_NUM2ULONG (SCM_ARG1, scm_length (list)), NULL);
+  for (i = 0; SCM_PAIRP (list); list = SCM_CDR (list), i++)
     {
-      if (guile_to_integer (gh_car (list), &n) != 0)
+      if (guile_to_integer (SCM_CAR (list), &n) != 0)
 	{
 	  guile_error ("%s: Integer expected in position %d", func, i);
 	  guile_global_error = -1;
@@ -545,6 +553,7 @@ guile_to_intarray (SCM list, char *func)
     }
   return array;
 }
+#undef FUNC_NAME
 
 /*
  * Extract an integer value from an option hash. Returns zero if it worked.
@@ -561,7 +570,7 @@ optionhash_extract_int (svz_hash_t *hash,
   SCM hvalue = optionhash_get (hash, key);
 
   /* Is there such an integer in the option-hash ? */
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       /* Nothing in hash, try to use default. */
       if (hasdef)
@@ -599,7 +608,7 @@ optionhash_extract_string (svz_hash_t *hash,
   char *str = NULL;
 
   /* Is there such a string in the option-hash ? */
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       /* Nothing in hash, try to use default. */
       if (hasdef)
@@ -621,7 +630,7 @@ optionhash_extract_string (svz_hash_t *hash,
       else
 	{
 	  *target = svz_strdup (str);
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
     }
   return err;
@@ -645,7 +654,7 @@ optionhash_cb_integer (char *server, void *arg, char *key, int *target,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
   
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -672,7 +681,7 @@ optionhash_cb_boolean (char *server, void *arg, char *key, int *target,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -701,7 +710,7 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
   SCM hvalue = optionhash_get (options, key);
 
   /* Is that integer array defined in the option-hash ? */
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -717,7 +726,7 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
       svz_array_t *array;
 
       /* Is it a valid list ? */
-      if (!gh_list_p (hvalue))
+      if (!SCM_LISTP (hvalue))
 	{
 	  guile_error ("%s: Integer array `%s' is not a valid list",
 		       server, key);
@@ -725,10 +734,10 @@ optionhash_cb_intarray (char *server, void *arg, char *key,
 	}
 
       /* Iterate over the list and build up the array of integers. */
-      array = svz_array_create (0);
-      for (i = 0; gh_pair_p (hvalue); hvalue = gh_cdr (hvalue), i++)
+      array = svz_array_create (0, NULL);
+      for (i = 0; SCM_PAIRP (hvalue); hvalue = SCM_CDR (hvalue), i++)
 	{
-	  if (guile_to_integer (gh_car (hvalue), &val))
+	  if (guile_to_integer (SCM_CAR (hvalue), &val))
 	    {
 	      guile_error ("%s: Invalid integer element #%d in "
 			   "integer array `%s'", server, i, key);
@@ -760,7 +769,7 @@ optionhash_cb_string (char *server, void *arg, char *key,
   SCM hvalue = optionhash_get (options, key);
   char *str;
 
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -777,7 +786,7 @@ optionhash_cb_string (char *server, void *arg, char *key,
     }
 
   *target = svz_strdup (str);
-  scm_must_free (str);
+  scm_c_free (str);
   return SVZ_ITEM_OK;
 }
 
@@ -790,7 +799,7 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -806,7 +815,7 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
       char *str;
 
       /* Check if this is really a list. */
-      if (!gh_list_p (hvalue))
+      if (!SCM_LISTP (hvalue))
 	{
 	  guile_error ("%s: String array `%s' is not a valid list",
 		       server, key);
@@ -814,10 +823,10 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
 	}
 
       /* Iterate over the list and build up the array of strings. */
-      array = svz_array_create (0);
-      for (i = 0; gh_pair_p (hvalue); hvalue = gh_cdr (hvalue), i++)
+      array = svz_array_create (0, svz_free);
+      for (i = 0; SCM_PAIRP (hvalue); hvalue = SCM_CDR (hvalue), i++)
 	{
-	  if (NULL == (str = guile_to_string (gh_car (hvalue))))
+	  if (NULL == (str = guile_to_string (SCM_CAR (hvalue))))
 	    {
 	      guile_error ("%s: Invalid string element #%d in "
 			   "string array `%s'", server, i, key);
@@ -825,14 +834,12 @@ optionhash_cb_strarray (char *server, void *arg, char *key,
 	      continue;
 	    }
 	  svz_array_add (array, svz_strdup (str));
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
 
       /* Free the string array so far. */
       if (err)
 	{
-	  svz_array_foreach (array, str, i)
-	    svz_free (str);
 	  svz_array_destroy (array);
 	  return SVZ_ITEM_FAILED;
 	}
@@ -852,7 +859,7 @@ optionhash_cb_hash (char *server, void *arg, char *key,
   svz_hash_t *options = arg;
   SCM hvalue = optionhash_get (options, key);
 
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -867,7 +874,7 @@ optionhash_cb_hash (char *server, void *arg, char *key,
       svz_hash_t *hash;
 
       /* Unpack again... Whysoever. */
-      if (!gh_list_p (hvalue))
+      if (!SCM_LISTP (hvalue))
 	{
 	  err = -1;
 	  guile_error ("%s: Not a valid list for hash (alist) `%s'",
@@ -877,20 +884,20 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 
       /* Iterate the alist. */
       hash = svz_hash_create (7);
-      for (i = 0; gh_pair_p (hvalue); hvalue = gh_cdr (hvalue), i++)
+      for (i = 0; SCM_PAIRP (hvalue); hvalue = SCM_CDR (hvalue), i++)
 	{
-	  SCM k, v, pair = gh_car (hvalue);
+	  SCM k, v, pair = SCM_CAR (hvalue);
 	  char *str, *keystr, *valstr;
 
-	  if (!gh_pair_p (pair))
+	  if (!SCM_PAIRP (pair))
 	    {
 	      err = -1;
 	      guile_error ("%s: Element #%d of hash (alist) `%s' "
 			   "is not a pair", server, i, key);
 	      continue;
 	    }
-	  k = gh_car (pair);
-	  v = gh_cdr (pair);
+	  k = SCM_CAR (pair);
+	  v = SCM_CDR (pair);
 	     
 	  /* Obtain key character string. */
 	  if (NULL == (str = guile_to_string (k)))
@@ -904,7 +911,7 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 	  else
 	    {
 	      keystr = svz_strdup (str);
-	      scm_must_free (str);
+	      scm_c_free (str);
 	    }
 
 	  /* Obtain value character string. */
@@ -919,7 +926,7 @@ optionhash_cb_hash (char *server, void *arg, char *key,
 	  else
 	    {
 	      valstr = svz_strdup (str);
-	      scm_must_free (str);
+	      scm_c_free (str);
 	    }
 
 	  /* Add to hash if key and value look good. */
@@ -958,7 +965,7 @@ optionhash_cb_portcfg (char *server, void *arg, char *key,
   char *str;
 
   /* Is the requested port configuration defined ? */
-  if (gh_eq_p (hvalue, SCM_UNSPECIFIED))
+  if (SCM_EQ_P (hvalue, SCM_UNSPECIFIED))
     {
       if (hasdef)
 	return SVZ_ITEM_DEFAULT_ERRMSG;
@@ -979,12 +986,12 @@ optionhash_cb_portcfg (char *server, void *arg, char *key,
   if ((port = svz_portcfg_get (str)) == NULL)
     {
       guile_error ("%s: No such port configuration: `%s'", server, str);
-      scm_must_free (str);
+      scm_c_free (str);
       return SVZ_ITEM_FAILED;
     }
 
   /* Duplicate this port configuration. */
-  scm_must_free (str);
+  scm_c_free (str);
   *target = svz_portcfg_dup (port);
   return SVZ_ITEM_OK;
 }
@@ -1073,7 +1080,7 @@ guile_define_server (SCM name, SCM args)
   svz_snprintf (txt, 256, "defining server `%s'", servername);
 
   /* Extract options if any. */
-  if (gh_eq_p (args, SCM_UNDEFINED))
+  if (SCM_UNBNDP (args))
     options = svz_hash_create (4);
   else if (NULL == (options = guile_to_optionhash (args, txt, 0)))
     FAIL (); /* Message already emitted. */
@@ -1130,7 +1137,7 @@ guile_define_server (SCM name, SCM args)
  out:
   svz_free (txt);
   svz_free (servertype);
-  scm_must_free (servername);
+  scm_c_free (servername);
   optionhash_destroy (options);
   guile_global_error |= err;
   return err ? SCM_BOOL_T : SCM_BOOL_F;
@@ -1247,10 +1254,10 @@ guile_define_port (SCM name, SCM args)
 	  cfg->pipe_recv.gid = (unsigned int) -1;
 	  cfg->pipe_recv.uid = (unsigned int) -1;
 	  cfg->pipe_recv.perm = (unsigned int) -1;
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
       /* Create local optionhash for receiving pipe direction. */
-      else if (gh_eq_p (p, SCM_UNSPECIFIED))
+      else if (SCM_EQ_P (p, SCM_UNSPECIFIED))
 	{
 	  guile_error ("%s: You have to define a pipe called `%s'",
 		       portname, PORTCFG_RECV);
@@ -1278,9 +1285,9 @@ guile_define_port (SCM name, SCM args)
 	  cfg->pipe_send.gid = (unsigned int) -1;
 	  cfg->pipe_send.uid = (unsigned int) -1;
 	  cfg->pipe_send.perm = (unsigned int) -1;
-	  scm_must_free (str);
+	  scm_c_free (str);
 	}
-      else if (gh_eq_p (p, SCM_UNSPECIFIED))
+      else if (SCM_EQ_P (p, SCM_UNSPECIFIED))
 	{
 	  guile_error ("%s: You have to define a pipe called `%s'",
 		       portname, PORTCFG_SEND);
@@ -1303,7 +1310,7 @@ guile_define_port (SCM name, SCM args)
 		   proto, portname);
       FAIL ();
     }
-  scm_must_free (proto);
+  scm_c_free (proto);
 
   /* Access the send and receive buffer sizes. */
   err |= optionhash_extract_int (options, PORTCFG_SEND_BUFSIZE, 1, 0,
@@ -1321,12 +1328,12 @@ guile_define_port (SCM name, SCM args)
     {
       SCM list;
       list = optionhash_get (options, PORTCFG_DENY);
-      if (!gh_eq_p (list, SCM_UNSPECIFIED))
+      if (!SCM_EQ_P (list, SCM_UNSPECIFIED))
 	cfg->deny = guile_to_strarray (list, FUNC_NAME);
       else
 	cfg->deny = NULL;
       list = optionhash_get (options, PORTCFG_ALLOW);
-      if (!gh_eq_p (list, SCM_UNSPECIFIED))
+      if (!SCM_EQ_P (list, SCM_UNSPECIFIED))
 	cfg->allow = guile_to_strarray (list, FUNC_NAME);
       else
 	cfg->allow = NULL;
@@ -1356,7 +1363,7 @@ guile_define_port (SCM name, SCM args)
  out:
   if (err)
     svz_portcfg_destroy (cfg);
-  scm_must_free (portname);
+  scm_c_free (portname);
   optionhash_destroy (options);
   guile_global_error |= err;
   return err ? SCM_BOOL_F : SCM_BOOL_T;
@@ -1413,8 +1420,8 @@ guile_bind_server (SCM port, SCM server)
     }
 
  out:
-  scm_must_free (portname);
-  scm_must_free (servername);
+  scm_c_free (portname);
+  scm_c_free (servername);
   guile_global_error |= err;
   return err ? SCM_BOOL_F : SCM_BOOL_T;
 }
@@ -1435,8 +1442,9 @@ guile_strarray_to_guile (svz_array_t *array)
 
   /* Go through all the strings and add these to a guile list. */
   for (list = SCM_EOL, i = 0; i < svz_array_size (array); i++)
-    list = gh_cons (gh_str02scm ((char *) svz_array_get (array, i)), list);
-  return gh_reverse (list);
+    list = scm_cons (scm_makfrom0str ((char *) svz_array_get (array, i)), 
+		     list);
+  return scm_reverse (list);
 }
 
 /*
@@ -1454,8 +1462,8 @@ guile_intarray_to_guile (svz_array_t *array)
 
   /* Go through all the strings and add these to a guile list. */
   for (list = SCM_EOL, i = 0; i < svz_array_size (array); i++)
-    list = gh_cons (gh_long2scm ((long) svz_array_get (array, i)), list);
-  return gh_reverse (list);
+    list = scm_cons (scm_long2num ((long) svz_array_get (array, i)), list);
+  return scm_reverse (list);
 }
 
 /*
@@ -1470,9 +1478,9 @@ guile_hash_to_guile (svz_hash_t *hash)
   
   svz_hash_foreach_key (hash, key, n)
     {
-      pair = gh_cons (gh_str02scm (key[n]),
-		      gh_str02scm ((char *) svz_hash_get (hash, key[n])));
-      alist = gh_cons (pair, alist);
+      pair = scm_cons (scm_makfrom0str (key[n]),
+		       scm_makfrom0str ((char *) svz_hash_get (hash, key[n])));
+      alist = scm_cons (pair, alist);
     }
   return alist;
 }
@@ -1498,19 +1506,14 @@ guile_access_interfaces (SCM args)
 
   /* First create a array of strings containing the ip addresses of each
      local network interface and put them into a guile list. */
-  array = svz_array_create (0);
+  array = svz_array_create (0, svz_free);
   svz_interface_foreach (ifc, n)
     svz_array_add (array, svz_strdup (svz_inet_ntoa (ifc->ipaddr)));
   list = guile_strarray_to_guile (array);
-  if (array)
-    {
-      svz_array_foreach (array, str, n)
-	svz_free (str);
-      svz_array_destroy (array);
-    }
+  svz_array_destroy (array);
 
   /* Is there an argument given to the guile function ? */
-  if (!gh_eq_p (args, SCM_UNDEFINED))
+  if (!SCM_UNBNDP (args))
     {
       svz_interface_free ();
       if ((array = guile_to_strarray (args, FUNC_NAME)) != NULL)
@@ -1527,8 +1530,6 @@ guile_access_interfaces (SCM args)
 	      sprintf (description, "guile interface %d", n);
 	      svz_interface_add (n, description, addr.sin_addr.s_addr);
 	    }
-	  svz_array_foreach (array, str, n)
-	    svz_free (str);
 	  svz_array_destroy (array);
 	}
     }
@@ -1538,9 +1539,9 @@ guile_access_interfaces (SCM args)
 #undef FUNC_NAME
 
 /*
- * Make the search path for the serveez core library accessable for guile.
+ * Make the search path for the Serveez core library accessable for guile.
  * Returns a list a each path as previously defined. Can override the current
- * definition of this load path. The load path is used to tell serveez where
+ * definition of this load path. The load path is used to tell Serveez where
  * it can find additional server modules.
  */
 #define FUNC_NAME "serveez-load-path"
@@ -1549,22 +1550,15 @@ guile_access_loadpath (SCM args)
 {
   SCM list;
   svz_array_t *paths = svz_dynload_path_get ();
-  int n;
-  char *str;
 
   GUILE_PRECALL ();
 
   /* Create a guile list containing each search path. */
   list = guile_strarray_to_guile (paths);
-  if (paths)
-    {
-      svz_array_foreach (paths, str, n)
-	svz_free (str);
-      svz_array_destroy (paths);
-    }
+  svz_array_destroy (paths);
   
   /* Set the load path if argument is given. */
-  if (!gh_eq_p (args, SCM_UNDEFINED))
+  if (!SCM_UNBNDP (args))
     {
       if ((paths = guile_to_strarray (args, FUNC_NAME)) != NULL)
 	svz_dynload_path_set (paths);
@@ -1585,30 +1579,40 @@ SCM cfunc (SCM arg) {                          \
   if ((str = guile_to_string (arg)) == NULL)   \
     return SCM_BOOL_F;                         \
   if (!(expression)) {                         \
-    scm_must_free (str);                       \
+    scm_c_free (str);                          \
     return SCM_BOOL_F; }                       \
-  scm_must_free (str);                         \
+  scm_c_free (str);                            \
   return SCM_BOOL_T;                           \
 }
 
-/* Guile function checking for a valid port configuration. */
+/* Returns @code{#t} if the given string @var{name} corresponds with a 
+   registered port configuration, otherwise the procedure returns 
+   @code{#f}. */
 #define FUNC_NAME "serveez-port?"
 MAKE_STRING_CHECKER (guile_check_port, svz_portcfg_get (str) != NULL)
 #undef FUNC_NAME
 
-/* Guile function checking for a valid server. */
+/* Checks whether the given string @var{name} corresponds with an 
+   instantiated server name and returns @code{#t} if so. */
 #define FUNC_NAME "serveez-server?"
 MAKE_STRING_CHECKER (guile_check_server, svz_server_get (str) != NULL)
 #undef FUNC_NAME
 
+/* This procedure checks whether the given string @var{name} is a valid
+   server type prefix known in Serveez and returns @code{#t} if so. 
+   Otherwise it returns @code{#f}. */
+#define FUNC_NAME "serveez-servertype?"
+MAKE_STRING_CHECKER (guile_check_stype, svz_servertype_get (str, 0) != NULL)
+#undef FUNC_NAME
+
 /*
  * Create an accessor function to read and write a C int variable.
  */
 #define MAKE_INT_ACCESSOR(cfunc, cvar)                       \
 static SCM cfunc (SCM args) {                                \
-  SCM value = gh_int2scm (cvar); int n;                      \
+  SCM value = scm_int2num (cvar); int n;                     \
   GUILE_PRECALL ();                                          \
-  if (!gh_eq_p (args, SCM_UNDEFINED)) {                      \
+  if (!SCM_UNBNDP (args)) {                                  \
     if (guile_to_integer (args, &n)) {                       \
       guile_error ("%s: Invalid integer value", FUNC_NAME);  \
       guile_global_error = -1;                               \
@@ -1621,16 +1625,16 @@ static SCM cfunc (SCM args) {                                \
  */
 #define MAKE_STRING_ACCESSOR(cfunc, cvar)                    \
 static SCM cfunc (SCM args) {                                \
-  SCM value = gh_str02scm (cvar); char *str;                 \
+  SCM value = scm_makfrom0str (cvar); char *str;             \
   GUILE_PRECALL ();                                          \
-  if (!gh_eq_p (args, SCM_UNDEFINED)) {                      \
+  if (!SCM_UNBNDP (args)) {                                  \
     if (NULL == (str = guile_to_string (args))) {            \
       guile_error ("%s: Invalid string value", FUNC_NAME);   \
       guile_global_error = -1;                               \
     } else {                                                 \
       svz_free (cvar);                                       \
       cvar = svz_strdup (str);                               \
-      scm_must_free (str);                                   \
+      scm_c_free (str);                                      \
     } }                                                      \
   return value;                                              \
 }
@@ -1658,27 +1662,28 @@ static void
 guile_init (void)
 {
   /* define some variables */
-  gh_define ("serveez-version", gh_str02scm (svz_version));
-  gh_define ("guile-version", scm_version ());
-  gh_define ("have-debug", gh_bool2scm (svz_have_debug));
-  gh_define ("have-Win32", gh_bool2scm (svz_have_Win32));
-  gh_define ("have-floodprotect", gh_bool2scm (svz_have_floodprotect));
+  scm_c_define ("serveez-version", scm_makfrom0str (svz_version));
+  scm_c_define ("guile-version", scm_version ());
+  scm_c_define ("have-debug", SCM_BOOL (svz_have_debug));
+  scm_c_define ("have-Win32", SCM_BOOL (svz_have_Win32));
+  scm_c_define ("have-floodprotect", SCM_BOOL (svz_have_floodprotect));
 
   /* export accessors for global variables (read/write capable) */
-  gh_new_procedure ("serveez-verbosity", guile_access_verbosity, 0, 1, 0);
-  gh_new_procedure ("serveez-maxsockets", guile_access_maxsockets, 0, 1, 0);
-  gh_new_procedure ("serveez-passwd", guile_access_passwd, 0, 1, 0);
-  gh_new_procedure ("serveez-interfaces", guile_access_interfaces, 0, 1, 0);
-  gh_new_procedure ("serveez-loadpath", guile_access_loadpath, 0, 1, 0);
+  scm_c_define_gsubr ("serveez-verbosity", 0, 1, 0, guile_access_verbosity);
+  scm_c_define_gsubr ("serveez-maxsockets", 0, 1, 0, guile_access_maxsockets);
+  scm_c_define_gsubr ("serveez-passwd", 0, 1, 0, guile_access_passwd);
+  scm_c_define_gsubr ("serveez-interfaces", 0, 1, 0, guile_access_interfaces);
+  scm_c_define_gsubr ("serveez-loadpath", 0, 1, 0, guile_access_loadpath);
 
   /* export some new procedures */
-  gh_new_procedure ("define-port!", guile_define_port, 2, 0, 0);
-  gh_new_procedure ("define-server!", guile_define_server, 1, 1, 0);
-  gh_new_procedure ("bind-server!", guile_bind_server, 2, 0, 0);
+  scm_c_define_gsubr ("define-port!", 2, 0, 0, guile_define_port);
+  scm_c_define_gsubr ("define-server!", 1, 1, 0, guile_define_server);
+  scm_c_define_gsubr ("bind-server!", 2, 0, 0, guile_bind_server);
 
   /* export checker functions */
-  gh_new_procedure ("serveez-port?", guile_check_port, 1, 0, 0);
-  gh_new_procedure ("serveez-server?", guile_check_server, 1, 0, 0);
+  scm_c_define_gsubr ("serveez-port?", 1, 0, 0, guile_check_port);
+  scm_c_define_gsubr ("serveez-server?", 1, 0, 0, guile_check_server);
+  scm_c_define_gsubr ("serveez-servertype?", 1, 0, 0, guile_check_stype);
 
 #if ENABLE_GUILE_SERVER
   guile_server_init ();
@@ -1695,30 +1700,38 @@ guile_exception (void *data, SCM tag, SCM args)
   /* FIXME: current-load-port is not defined in this state. Why ? */
   char *str = guile_to_string (tag);
   guile_error ("Exception due to `%s'", str);
-  scm_must_free (str);
+  scm_c_free (str);
 
   /* `tag' contains internal exception name */
   scm_puts ("guile-error: ", scm_current_error_port ());
 
   /* on quit/exit */
-  if (gh_null_p (args))
+  if (SCM_NULLP (args))
     {
       scm_display (tag, scm_current_error_port ());
       scm_puts ("\n", scm_current_error_port ());
       return SCM_BOOL_F;
     }
 
-  if (!gh_eq_p (gh_car (args), SCM_BOOL_F))
+  if (!SCM_FALSEP (SCM_CAR (args)))
     {
-      scm_display (gh_car (args), scm_current_error_port ());
+      scm_display (SCM_CAR (args), scm_current_error_port ());
       scm_puts (": ", scm_current_error_port ());
     }
-  scm_display_error_message (gh_car (gh_cdr (args)), 
-			     gh_car (gh_cdr (gh_cdr (args))), 
+  scm_display_error_message (SCM_CAR (SCM_CDR (args)), 
+			     SCM_CAR (SCM_CDR (SCM_CDR (args))), 
 			     scm_current_error_port ());
   return SCM_BOOL_F;
 }
 
+/* Wrapper function for the file loader exception handler. */
+static SCM
+guile_eval_file (void *data)
+{
+  char *file = (char *) data;
+  return scm_c_primitive_load (file);
+}
+
 /*
  * Get server settings from the file @var{cfgfile} and instantiate servers 
  * as needed. Return non-zero on errors.
@@ -1726,11 +1739,23 @@ guile_exception (void *data, SCM tag, SCM args)
 int
 guile_load_config (char *cfgfile)
 {
+  SCM ret;
   guile_global_error = 0;
   guile_init ();
 
-  if (gh_eq_p (gh_eval_file_with_catch (cfgfile, guile_exception), SCM_BOOL_F))
+  ret = scm_internal_catch (SCM_BOOL_T,
+			    (scm_catch_body_t) guile_eval_file, 
+			    (void *) cfgfile, 
+			    (scm_catch_handler_t) guile_exception,
+			    (void *) cfgfile);
+
+  if (SCM_FALSEP (ret))
     guile_global_error = -1; 
 
+  /* Kick the gc now since no guile is involved anymore unless a guile
+   * server is implemented.
+   */
+  scm_gc ();
+
   return guile_global_error ? -1 : 0;
 }
