@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-core.c,v 1.38 2001/07/05 21:35:26 ela Exp $
+ * $Id: http-core.c,v 1.42 2001/08/03 18:09:04 ela Exp $
  *
  */
 
@@ -83,7 +83,7 @@ http_header_t http_header;
  * Handle and function definitions for the NetApi interface. 
  */
 typedef NET_API_STATUS (__stdcall *GetUserInfoProc) (WCHAR *, WCHAR *,
-						     DWORD, byte **);
+						     DWORD, LPBYTE *);
 typedef NET_API_STATUS (__stdcall *FreeUserInfoProc) (void *);
 static FreeUserInfoProc FreeUserInfo = NULL;
 static GetUserInfoProc GetUserInfo = NULL;
@@ -225,7 +225,7 @@ http_userdir (svz_socket_t *sock, char *uri)
 
 /*
  * When the http server has been configured to invoke identd request
- * this function is called for each client connection after successfull
+ * this function is called for each client connection after successful
  * identification.
  */
 int
@@ -450,7 +450,7 @@ http_set_header (char *response)
 
 /*
  * Create a http content range if the given line specifies a valid one.
- * Return zero on succes and -1 on errors.
+ * Return zero on success and -1 on errors.
  */
 int
 http_get_range (char *line, http_range_t *range)
@@ -697,8 +697,8 @@ http_clf_date (time_t t)
 	   tm->tm_mday, months[tm->tm_mon], tm->tm_year + 1900,
 	   tm->tm_hour, tm->tm_min, tm->tm_sec,
 	   timezone > 0 ? '+' : '-',
-	   timezone > 0 ? timezone / 3600 : - timezone / 3600,
-	   timezone > 0 ? (timezone / 60) % 60 : - (timezone / 60) % 60);
+	   timezone > 0 ? timezone / 3600 : -timezone / 3600,
+	   timezone > 0 ? (timezone / 60) % 60 : -(timezone / 60) % 60);
   return date;
 }
 
@@ -901,25 +901,6 @@ http_process_uri (char *uri)
     }
 }
 
-/*
- * This function frees all the content type definitions.
- */
-void
-http_free_types (http_config_t *cfg)
-{
-#if 0
-  /* FIXME: Is that still neccessary ? */
-  char **type;
-  int n;
-
-  svz_hash_foreach_value (cfg->types, type, n)
-    svz_free (type[n]);
-      
-  svz_hash_destroy (cfg->types);
-  cfg->types = NULL;
-#endif
-}
-
 /*
  * This routine gets all available content types from a given
  * file which should have kind of "/etc/mime.types"s format.
@@ -1031,8 +1012,6 @@ http_find_content_type (svz_socket_t *sock, char *file)
  * absolute file/path name. The given argument will be reallocated
  * if necessary.
  */
-#define MAX_DIR_LEN 1024
-
 char *
 http_absolute_file (char *file)
 {
@@ -1053,18 +1032,10 @@ http_absolute_file (char *file)
     }
 
   /* save the filename within a buffer */
-  savefile = svz_malloc (strlen (p) + 1);
-  strcpy (savefile, p);
+  savefile = svz_strdup (p);
 
   /* get current work directory */
-  savedir = svz_malloc (MAX_DIR_LEN);
-  if ((getcwd (savedir, MAX_DIR_LEN)) == NULL)
-    {
-      svz_log (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
-      svz_free (savefile);
-      svz_free (savedir);
-      return file;
-    }
+  savedir = svz_getcwd ();
   
   /* 
    * If there was no path separator in the filename then just concate
@@ -1072,9 +1043,9 @@ http_absolute_file (char *file)
    */
   if (!have_path)
     {
+      savedir = svz_realloc (savedir, strlen (savedir) + strlen (file) + 2);
       strcat (savedir, "/");
       strcat (savedir, savefile);
-      savedir = svz_realloc (savedir, strlen (savedir) + 1);
       svz_free (file);
       return savedir;
     }
@@ -1095,20 +1066,12 @@ http_absolute_file (char *file)
   *p = '/';
 
   /* get now the current work directory */
-  dir = svz_malloc (MAX_DIR_LEN);
-  if ((getcwd (dir, MAX_DIR_LEN)) == NULL)
-    {
-      svz_log (LOG_ERROR, "getcwd: %s\n", SYS_ERROR);
-      svz_free (dir);
-      svz_free (savefile);
-      svz_free (savedir);
-      return file;
-    }
+  dir = svz_getcwd ();
 
   /* concate new work directory with given filename */
+  dir = svz_realloc (dir, strlen (dir) + strlen (savefile) + 2);
   strcat (dir, "/");
   strcat (dir, savefile);
-  dir = svz_realloc (dir, strlen (dir) + 1);
   svz_free (savefile);
   svz_free (file);
 
