@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: http-core.h,v 1.12 2001/05/19 23:04:57 ela Exp $
+ * $Id: http-core.h,v 1.14 2001/09/27 15:47:36 ela Exp $
  *
  */
 
@@ -133,7 +133,6 @@ int http_keep_alive (svz_socket_t *sock);
 void http_check_keepalive (svz_socket_t *sock);
 
 int http_read_types (http_config_t *cfg);
-void http_free_types (http_config_t *cfg);
 char *http_find_content_type (svz_socket_t *sock, char *file);
 
 int http_parse_property (svz_socket_t *sock, char *request, char *end);
@@ -154,11 +153,7 @@ char *http_clf_date (time_t t);
 void http_set_header (char *response);
 int http_send_header (svz_socket_t *sock);
 void http_reset_header (void);
-#ifndef __STDC__
-int http_add_header ();
-#else
-void http_add_header (const char *fmt, ...);
-#endif
+void http_add_header __P ((const char *fmt, ...));
 
 #ifdef __MINGW32__
 void http_start_netapi (void);
