@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: codec.c,v 1.2 2001/10/08 13:02:54 ela Exp $
+ * $Id: codec.c,v 1.8 2001/11/23 13:18:39 ela Exp $
  *
  */
 
@@ -38,10 +38,18 @@
 #include "libserveez/socket.h"
 #include "libserveez/codec/codec.h"
 
+/* Return text representation for codec type. */
+#define SVZ_CODEC_TYPE_TEXT(codec)                          \
+  (((codec)->type == SVZ_CODEC_DECODER) ? "decoder" :       \
+   ((codec)->type == SVZ_CODEC_ENCODER) ? "encoder" : NULL)
+
 /* Include codec headers if any. */
 #if HAVE_ZLIB
 #include "libserveez/codec/gzlib.h"
 #endif
+#if HAVE_BZ2LIB
+#include "libserveez/codec/bzip2.h"
+#endif
 
 /* Collection of available encoder and decoder provided by the libcodec
    archive or by external (shared) libraries. */
@@ -103,6 +111,10 @@ svz_codec_init (void)
   svz_codec_register (&zlib_encoder);
   svz_codec_register (&zlib_decoder);
 #endif /* HAVE_ZLIB */
+#if HAVE_BZ2LIB
+  svz_codec_register (&bzip2_encoder);
+  svz_codec_register (&bzip2_decoder);
+#endif /* HAVE_BZ2LIB */
   return 0;
 }
 
@@ -158,10 +170,10 @@ svz_codec_register (svz_codec_t *codec)
 
   /* Add this codec to the list of known codecs. */
   if (svz_codecs == NULL)
-    svz_codecs = svz_array_create (2);
+    svz_codecs = svz_array_create (2, NULL);
   svz_array_add (svz_codecs, codec);
   svz_log (LOG_NOTICE, "registered `%s' %s\n", codec->description,
-	   codec->type == SVZ_CODEC_ENCODER ? "encoder" : "decoder");
+	   SVZ_CODEC_TYPE_TEXT (codec));
   return 0;
 }
 
@@ -188,7 +200,7 @@ svz_codec_unregister (svz_codec_t *codec)
 	{
 	  svz_array_del (svz_codecs, i);
 	  svz_log (LOG_NOTICE, "unregistered `%s' %s\n", codec->description,
-		   codec->type == SVZ_CODEC_ENCODER ? "encoder" : "decoder");
+		   SVZ_CODEC_TYPE_TEXT (codec));
 	  return 0;
 	}
     }
@@ -198,6 +210,27 @@ svz_codec_unregister (svz_codec_t *codec)
   return -1;
 }
 
+/* Print a text representation of a codec's current ratio in percent
+   if possible. */
+void
+svz_codec_ratio (svz_codec_t *codec, svz_codec_data_t *data)
+{
+  unsigned long in = 0, out = 0;
+
+  if (codec->ratio == NULL)
+    return;
+  if (codec->ratio (data, &in, &out) == SVZ_CODEC_OK)
+    {
+      if (in != 0)
+	svz_log (LOG_NOTICE, "%s: %s ratio is %lu.%02lu%%\n",
+		 codec->description, SVZ_CODEC_TYPE_TEXT (codec),
+		 out * 100UL / in, (out * 10000UL / in) % 100UL);
+      else
+	svz_log (LOG_NOTICE, "%s: %s ratio is infinite\n",
+		 codec->description, SVZ_CODEC_TYPE_TEXT (codec));
+    }
+}
+
 /* The following four (4) macros are receive buffer switcher used in order
    to apply the output buffer of the codec to the receive buffer of a socket
    structure and to revert these changes. */
@@ -257,9 +290,10 @@ svz_codec_sock_receive_setup (svz_socket_t *sock, svz_codec_t *codec)
     return 0;
 
   /* Setup internal codec data. */
-  data = svz_malloc (sizeof (svz_codec_data_t));
+  data = svz_calloc (sizeof (svz_codec_data_t));
   data->codec = codec;
   data->flag = SVZ_CODEC_INIT;
+  data->state = SVZ_CODEC_NONE;
   data->config = data->data = NULL;
   sock->recv_codec = data;
 
@@ -287,6 +321,9 @@ svz_codec_sock_receive_setup (svz_socket_t *sock, svz_codec_t *codec)
       svz_codec_sock_recv_revert (sock);
       return -1;
     }
+  data->state |= SVZ_CODEC_READY;
+  svz_log (LOG_NOTICE, "%s: %s initialized\n", codec->description,
+	   SVZ_CODEC_TYPE_TEXT (codec));
   return 0;
 }
 
@@ -305,6 +342,12 @@ svz_codec_sock_receive (svz_socket_t *sock)
   svz_codec_t *codec = data->codec;
   int ret;
 
+  /* Check internal state of codec first. */
+  if (!(data->state & SVZ_CODEC_READY))
+    {
+      return 0;
+    }
+
   /* Run the encoder / decoder of the applied codec. */
   data->flag = SVZ_CODEC_CODE;
   if (sock->flags & SOCK_FLAG_FLUSH)
@@ -313,7 +356,7 @@ svz_codec_sock_receive (svz_socket_t *sock)
   while ((ret = codec->code (data)) == SVZ_CODEC_MORE_OUT)
     {
       /* Resize output buffer if necessary. */
-      data->flag = SVZ_CODEC_FLUSH;
+      data->flag |= SVZ_CODEC_FLUSH;
       data->out_size *= 2;
       data->out_buffer = svz_realloc (data->out_buffer, data->out_size);
     }
@@ -327,11 +370,18 @@ svz_codec_sock_receive (svz_socket_t *sock)
       return -1;
 
     case SVZ_CODEC_FINISHED: /* Codec finished. */
+      svz_codec_ratio (codec, data);
       if (codec->finalize (data) != SVZ_CODEC_OK)
 	{
 	  svz_log (LOG_ERROR, "%s: finalize: %s\n", 
 		   codec->description, codec->error (data));
 	}
+      else
+	{
+	  data->state &= ~SVZ_CODEC_READY;
+	  svz_log (LOG_NOTICE, "%s: %s finalized\n", 
+		   codec->description, SVZ_CODEC_TYPE_TEXT (codec));
+	}
       break;
 
     case SVZ_CODEC_OK:       /* No error. */
@@ -421,9 +471,10 @@ svz_codec_sock_send_setup (svz_socket_t *sock, svz_codec_t *codec)
     return 0;
 
   /* Setup internal codec data. */
-  data = svz_malloc (sizeof (svz_codec_data_t));
+  data = svz_calloc (sizeof (svz_codec_data_t));
   data->codec = codec;
   data->flag = SVZ_CODEC_INIT;
+  data->state = SVZ_CODEC_NONE;
   data->config = data->data = NULL;
   sock->send_codec = data;
 
@@ -451,6 +502,9 @@ svz_codec_sock_send_setup (svz_socket_t *sock, svz_codec_t *codec)
       svz_codec_sock_send_revert (sock);
       return -1;
     }
+  data->state |= SVZ_CODEC_READY;
+  svz_log (LOG_NOTICE, "%s: %s initialized\n", codec->description,
+	   SVZ_CODEC_TYPE_TEXT (codec));
   return 0;
 }
 
@@ -468,6 +522,12 @@ svz_codec_sock_send (svz_socket_t *sock)
   svz_codec_t *codec = data->codec;
   int ret;
 
+  /* Check internal state of codec first. */
+  if (!(data->state & SVZ_CODEC_READY))
+    {
+      return 0;
+    }
+
   /* Run the encoder / decoder of the applied codec. */
   data->flag = SVZ_CODEC_CODE;
   if (sock->flags & SOCK_FLAG_FLUSH)
@@ -476,7 +536,7 @@ svz_codec_sock_send (svz_socket_t *sock)
   while ((ret = codec->code (data)) == SVZ_CODEC_MORE_OUT)
     {
       /* Resize output buffer if necessary. */
-      data->flag = SVZ_CODEC_FLUSH;
+      data->flag |= SVZ_CODEC_FLUSH;
       data->out_size *= 2;
       data->out_buffer = svz_realloc (data->out_buffer, data->out_size);
     }
@@ -490,11 +550,18 @@ svz_codec_sock_send (svz_socket_t *sock)
       return -1;
 
     case SVZ_CODEC_FINISHED: /* Codec finished. */
+      svz_codec_ratio (codec, data);
       if (codec->finalize (data) != SVZ_CODEC_OK)
 	{
 	  svz_log (LOG_ERROR, "%s: finalize: %s\n", 
 		   codec->description, codec->error (data));
 	}
+      else
+	{
+	  data->state &= ~SVZ_CODEC_READY;
+	  svz_log (LOG_NOTICE, "%s: %s finalized\n", 
+		   codec->description, SVZ_CODEC_TYPE_TEXT (codec));
+	}
       break;
 
     case SVZ_CODEC_OK:       /* No error. */
@@ -542,14 +609,16 @@ svz_codec_sock_disconnect (svz_socket_t *sock)
   if ((data = (svz_codec_data_t *) sock->recv_codec) != NULL)
     {
       disconnected = data->disconnected_socket;
-      data->codec->finalize (data);
+      if (data->state & SVZ_CODEC_READY)
+	data->codec->finalize (data);
       svz_codec_sock_recv_revert (sock);
     }
   /* Check and release sending codec. */
   if ((data = (svz_codec_data_t *) sock->send_codec) != NULL)
     {
       disconnected = data->disconnected_socket;
-      data->codec->finalize (data);
+      if (data->state & SVZ_CODEC_READY)
+	data->codec->finalize (data);
       svz_codec_sock_send_revert (sock);
     }
 
@@ -558,3 +627,29 @@ svz_codec_sock_disconnect (svz_socket_t *sock)
     return disconnected (sock);
   return 0;
 }
+
+/* This routine can be used to detect a codec on the receive buffer of the
+   given socket structure @var{sock}. It returns a valid codec or @code{NULL}
+   if no codec could be detected. */
+svz_codec_t *
+svz_codec_sock_detect (svz_socket_t *sock)
+{
+  svz_codec_t *codec;
+  int i;
+
+  svz_array_foreach (svz_codecs, codec, i)
+    {
+      if (codec->detection_size > 0 && 
+	  sock->recv_buffer_fill >= codec->detection_size)
+	{
+	  if (memcmp (sock->recv_buffer,
+		      codec->detection, codec->detection_size) == 0)
+	    {
+	      svz_log (LOG_NOTICE, "%s: %s detected\n", codec->description,
+		       SVZ_CODEC_TYPE_TEXT (codec));
+	      return codec;
+	    }
+	}
+    }
+  return NULL;
+}
