@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: core.c,v 1.21 2001/09/15 16:10:58 ela Exp $
+ * $Id: core.c,v 1.25 2001/11/23 13:18:38 ela Exp $
  *
  */
 
@@ -161,6 +161,68 @@ svz_fd_cloexec (int fd)
   return 0;
 }
 
+/*
+ * This funtion creates an unnamed pair of connected sockets with the 
+ * specified protocol @var{proto}. The descriptors used in referencing the 
+ * new sockets are returned in desc[0] and desc[1]. The two sockets are 
+ * indistinguishable. Also make both of them non-blocking and 
+ * non-inheritable. Returns -1 on failure, otherwise zero.
+ */
+int
+svz_socket_create_pair (int proto, SOCKET desc[2])
+{
+#ifndef HAVE_SOCKETPAIR
+  svz_log (LOG_FATAL, "socketpair() not available\n");
+  return -1;
+#else
+
+  int stype, ptype;
+
+  /* Assign the appropriate socket type. */
+  switch (proto)
+    {
+    case PROTO_TCP:
+      stype = SOCK_STREAM;
+      ptype = IPPROTO_IP;
+      break;
+    case PROTO_UDP:
+      stype = SOCK_DGRAM;
+      ptype = IPPROTO_UDP;
+      break;
+    case PROTO_ICMP:
+      stype = SOCK_RAW;
+      ptype = IPPROTO_ICMP;
+      break;
+    case PROTO_RAW:
+      stype = SOCK_RAW;
+      ptype = IPPROTO_RAW;
+      break;
+    default:
+      stype = SOCK_STREAM;
+      ptype = IPPROTO_IP;
+      break;
+    }
+
+  /* Create the pair of sockets. */
+  if (socketpair (AF_UNIX, stype, ptype, desc) < 0)
+    {
+      svz_log (LOG_ERROR, "socketpair: %s\n", NET_ERROR);
+      return -1;
+    }
+
+  /* Make the sockets non-blocking and non-inheritable. */
+  if (svz_fd_nonblock (desc[0]) != 0 || svz_fd_nonblock (desc[1]) != 0 ||
+      svz_fd_cloexec (desc[0]) != 0 || svz_fd_cloexec (desc[1]) != 0)
+    {
+      closesocket (desc[0]);
+      closesocket (desc[1]);
+      return -1;
+    }
+
+  return 0;
+#endif /* HAVE_SOCKETPAIR */
+}
+
 /*
  * Create a new non-blocking socket which does not get inherited on 
  * @code{exec()}. The protocol is specified by @var{proto}. Return the
@@ -225,6 +287,30 @@ svz_socket_create (int proto)
   return sockfd;
 }
 
+/*
+ * Saves the socket type (like @code{SOCK_STREAM}, @code{SOCK_DGRAM}, etc.) 
+ * of the socket @var{fd} in the buffer pointed to by @var{type}. Returns
+ * zero on success.
+ */
+int
+svz_socket_type (SOCKET fd, int *type)
+{
+  int optval;
+  socklen_t optlen = sizeof (optval);
+
+  if (type)
+    {
+      if (getsockopt (fd, SOL_SOCKET, SO_TYPE, 
+		      (void *) &optval, &optlen) < 0)
+	{
+	  svz_log (LOG_ERROR, "getsockopt: %s\n", NET_ERROR);
+	  return -1;
+	}
+      *type = optval;
+    }
+  return 0;
+}
+
 /*
  * Connect the given socket descriptor @var{sockfd} to the host @var{host}
  * at the network port @var{port}. Return non-zero on errors.
@@ -368,6 +454,55 @@ svz_tcp_cork (SOCKET fd, int set)
   return 0;
 }
 
+/* M$-Windows compatibility definition. */
+#ifndef SOL_TCP
+#define SOL_TCP IPPROTO_TCP
+#endif
+
+/*
+ * Enable or disable the @code{TCP_NODELAY} setting for the given socket
+ * descriptor @var{fd} depending on the flag @var{set}. In fact its turns 
+ * the Nagle algorithm on or off. This means that packets are always sent 
+ * as soon as possible and no unnecessary delays are introduced. The 
+ * function saves the old setting if @var{old} is not @code{NULL}. Returns 
+ * zero on success, otherwise non-zero.
+ */
+int
+svz_tcp_nodelay (SOCKET fd, int set, int *old)
+{
+#ifdef TCP_NODELAY
+  int optval;
+  socklen_t optlen = sizeof (optval);
+
+  /* Get old setting if required. */
+  if (old != NULL)
+    {  
+      if (getsockopt (fd, SOL_TCP, TCP_NODELAY, 
+		      (void *) &optval, &optlen) < 0)
+	{
+	  svz_log (LOG_ERROR, "getsockopt: %s\n", NET_ERROR);
+	  return -1;
+	}
+      *old = optval ? 1 : 0;
+    }
+
+  /* Set the setting. */
+  optval = set ? 1 : 0;
+  if (setsockopt (fd, SOL_TCP, TCP_NODELAY, 
+		  (void *) &optval, sizeof (optval)) < 0)
+    {
+      svz_log (LOG_ERROR, "setsockopt: %s\n", NET_ERROR);
+      return -1;
+    }
+#else /* not TCP_NODELAY */
+  if (old)
+    {
+      *old = 0;
+    }
+#endif /* not TCP_NODELAY */
+  return 0;
+}
+
 /*
  * This function transmits data between one file descriptor and another 
  * where @var{in_fd} is the source and @var{out_fd} the destination. The
@@ -492,7 +627,7 @@ static void
 svz_file_add (int fd)
 {
   if (svz_files == NULL)
-    svz_files = svz_array_create (1);
+    svz_files = svz_array_create (1, NULL);
   svz_array_add (svz_files, SVZ_NUM2PTR (fd));
 }
 
