@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: icmp-socket.c,v 1.17 2001/09/19 09:49:18 ela Exp $
+ * $Id: icmp-socket.c,v 1.19 2001/11/29 23:41:43 raimi Exp $
  *
  */
 
@@ -488,6 +488,22 @@ svz_icmp_read_socket (svz_socket_t *sock)
   return 0;
 }
 
+/*
+ * Default reader for ICMP server sockets. Allocates necessary buffers and
+ * reverts to @code{svz_icmp_read_socket()}.
+ */
+int
+svz_icmp_lazy_read_socket (svz_socket_t *sock)
+{
+  svz_portcfg_t *port = sock->port;
+
+  svz_sock_resize_buffers (sock, port->send_buffer_size,
+			   port->recv_buffer_size);
+  sock->read_socket = svz_icmp_read_socket;
+
+  return sock->read_socket (sock);
+}
+
 /*
  * The default ICMP write callback is called whenever the socket 
  * descriptor has been @code{select()}'ed or @code{poll()}'ed to be ready for 
@@ -770,6 +786,7 @@ svz_icmp_connect (unsigned long host, unsigned short port,
   svz_sock_resize_buffers (sock, ICMP_BUF_SIZE, ICMP_BUF_SIZE);
   svz_sock_unique_id (sock);
   sock->sock_desc = sockfd;
+  sock->proto = PROTO_ICMP;
   sock->flags |= (SOCK_FLAG_SOCK | SOCK_FLAG_CONNECTED | SOCK_FLAG_FIXED);
   sock->itype = type;
   svz_sock_enqueue (sock);
