@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: pipe-socket.c,v 1.15 2001/07/05 21:35:26 ela Exp $
+ * $Id: pipe-socket.c,v 1.21 2001/09/17 21:58:53 ela Exp $
  *
  */
 
@@ -57,6 +57,59 @@
 #include "libserveez/server-core.h"
 #include "libserveez/pipe-socket.h"
 
+#ifdef __MINGW32__
+/* Some static data for the CancelIo() call. We need to load the symbol
+   dynamically because it is not available on all versions of Windows. */
+typedef BOOL (__stdcall * CancelIoProc) (HANDLE);
+static CancelIoProc CancelIoFunc = NULL;
+static HANDLE Kernel32Handle = NULL;
+#endif /* __MINGW32__ */
+
+/*
+ * Startup the pipe interface of the core API of serveez. Returns zero on
+ * success. Gets called from @code{svz_boot()} once. Do not use.
+ */
+int
+svz_pipe_startup (void)
+{
+#ifdef __MINGW32__
+  if ((Kernel32Handle = LoadLibrary ("KERNEL32.DLL")) == NULL)
+    {
+      svz_log (LOG_ERROR, "pipe: LoadLibrary: %s\n", SYS_ERROR);
+      return -1;
+    }
+
+  /* obtain CancelIo() function */
+  CancelIoFunc = (CancelIoProc) GetProcAddress (Kernel32Handle, "CancelIo");
+  if (CancelIoFunc == NULL)
+    {
+      svz_log (LOG_ERROR, "pipe: GetProcAddress: %s\n", SYS_ERROR);
+      FreeLibrary (Kernel32Handle);
+      Kernel32Handle = NULL;
+      return -1;
+    }
+#endif /* __MINGW32__ */
+  return 0;
+}
+
+/*
+ * Cleanup the pipe interface of the core API of serveez. Returns zero on
+ * success. Gets called from @code{svz_halt()} once. Do not use.
+ */
+int
+svz_pipe_cleanup (void)
+{
+#ifdef __MINGW32__
+  if (Kernel32Handle != NULL)
+    {
+      FreeLibrary (Kernel32Handle);
+      Kernel32Handle = NULL;
+      CancelIoFunc = NULL;
+    }
+#endif /* __MINGW32__ */
+  return 0;
+}
+
 /*
  * Return a newly allocated and setup to some defaults pipe structure.
  */
@@ -212,6 +265,16 @@ svz_pipe_disconnect (svz_socket_t *sock)
       if ((rsock = svz_sock_getreferrer (sock)) != NULL)
 	{
 #ifdef __MINGW32__
+	  /* cancel any pending I/O if necessary and possible */
+	  if (CancelIoFunc)
+	    {
+	      if (sock->flags & (SOCK_FLAG_READING | SOCK_FLAG_CONNECTING))
+		if (!CancelIoFunc (sock->pipe_desc[READ]))
+		  svz_log (LOG_ERROR, "CancelIo: %s\n", SYS_ERROR);
+	      if (sock->flags & (SOCK_FLAG_WRITING | SOCK_FLAG_CONNECTING))
+		if (!CancelIoFunc (sock->pipe_desc[WRITE]))
+		  svz_log (LOG_ERROR, "CancelIo: %s\n", SYS_ERROR);
+	    }
 
 	  /* just disconnect client pipes */
 	  if (!DisconnectNamedPipe (sock->pipe_desc[READ]))
@@ -219,7 +282,7 @@ svz_pipe_disconnect (svz_socket_t *sock)
 	  if (!DisconnectNamedPipe (sock->pipe_desc[WRITE]))
 	    svz_log (LOG_ERROR, "DisconnectNamedPipe: %s\n", SYS_ERROR);
 
-	  /* reinitialize the overlapped structure */
+	  /* reinitialize the overlapped structure of the listener */
 	  if (svz_os_version >= WinNT4x)
 	    {
 	      memset (sock->overlap[READ], 0, sizeof (OVERLAPPED));
@@ -268,7 +331,13 @@ svz_pipe_disconnect (svz_socket_t *sock)
   if (sock->flags & SOCK_FLAG_LISTENING)
     {
       if ((rsock = svz_sock_getreferrer (sock)) != NULL)
-	svz_sock_setreferrer (rsock, NULL);
+	{
+#ifdef __MINGW32__
+	  rsock->overlap[READ] = NULL;
+	  rsock->overlap[WRITE] = NULL;
+#endif /* __MINGW32__ */
+	  svz_sock_setreferrer (rsock, NULL);
+	}
 
 #ifndef __MINGW32__
 
@@ -321,9 +390,9 @@ svz_pipe_overlap (LPOVERLAPPED overlap)
 {
   if (overlap)
     {
-      printf ("Internal: %ld (0x%08X), InternalHigh: %ld (0x%08X)\n"
-	      "Offset: %ld (0x%08X), OffsetHigh: %ld (0x%08X)\n"
-	      "Event: %ld\n",
+      printf ("Internal: %ld (0x%08lX), InternalHigh: %ld (0x%08lX)\n"
+	      "Offset: %ld (0x%08lX), OffsetHigh: %ld (0x%08lX)\n"
+	      "Event: %p\n",
 	      overlap->Internal, overlap->Internal, overlap->InternalHigh,
 	      overlap->InternalHigh, overlap->Offset, overlap->Offset,
 	      overlap->OffsetHigh, overlap->OffsetHigh, overlap->hEvent);
@@ -341,46 +410,82 @@ svz_pipe_read_socket (svz_socket_t *sock)
 {
   int num_read, do_read;
 
-  /* 
-   * Read as much space is left in the receive buffer and return 
-   * zero if there is no more space.
-   */
+  /* Read as much space is left in the receive buffer and return 
+   * zero if there is no more space. */
   do_read = sock->recv_buffer_size - sock->recv_buffer_fill;
   if (do_read <= 0) 
     {
       svz_log (LOG_ERROR, "receive buffer overflow on pipe %d\n", 
 	       sock->pipe_desc[READ]);
-      
       if (sock->kicked_socket)
 	sock->kicked_socket (sock, 0);
-      
       return -1;
     }
 
 #ifdef __MINGW32__
-  /* check how many bytes could have been read from the pipe */
-  if (!PeekNamedPipe (sock->pipe_desc[READ], NULL, 0, 
-		      NULL, (DWORD *) &num_read, NULL))
+  /* Named pipes in Win32 cannot transfer more than 64KB at once. */
+  if (do_read > PIPE_MAX_READ)
+    do_read = PIPE_MAX_READ;
+
+  /* Use the PeekNamedPipe() call if there is no overlapped I/O in 
+     order to make the following ReadFile() non-blocking. */
+  if (sock->overlap[READ] == NULL)
     {
-      svz_log (LOG_ERROR, "pipe: PeekNamedPipe: %s\n", SYS_ERROR);
-      return -1;
+      /* Check how many bytes could have been read from the pipe without
+	 really reading them. */
+      if (!PeekNamedPipe (sock->pipe_desc[READ], NULL, 0, 
+			  NULL, (DWORD *) &num_read, NULL))
+	{
+	  svz_log (LOG_ERROR, "pipe: PeekNamedPipe: %s\n", SYS_ERROR);
+	  return -1;
+	}
+
+      /* Leave this function if there is no data within the pipe. */
+      if (num_read <= 0)
+	return 0;
+
+      /* Adjust number of bytes to read. */
+      if (do_read > num_read)
+	do_read = num_read;
     }
 
-  /* leave this function if there is no data within the pipe */
-  if (num_read <= 0)
-    return 0;
-
-  /* adjust number of bytes to read */
-  if (do_read > num_read)
-    do_read = num_read;
-
-  /* really read from pipe */
-  if (!ReadFile (sock->pipe_desc[READ],
-		 sock->recv_buffer + sock->recv_buffer_fill,
-		 do_read, (DWORD *) &num_read, NULL))
+  /* Try to get the result of the last ReadFile(). */
+  if (sock->flags & SOCK_FLAG_READING)
     {
-      svz_log (LOG_ERROR, "pipe: ReadFile: %s\n", SYS_ERROR);
-      return -1;
+      if (!GetOverlappedResult (sock->pipe_desc[READ], sock->overlap[READ], 
+                                (DWORD *) &num_read, FALSE))
+        {
+          if (GetLastError () != ERROR_IO_INCOMPLETE)
+            {
+              svz_log (LOG_ERROR, "pipe: GetOverlappedResult: %s\n", 
+		       SYS_ERROR);
+              return -1;
+            }
+	  return 0;
+        }
+
+      /* Schedule the pipe for the ReadFile() call again. */
+      else
+	{
+	  sock->recv_pending = 0;
+	  sock->flags &= ~SOCK_FLAG_READING;
+	}
+    }
+  /* Really read from the pipe. */
+  else if (!ReadFile (sock->pipe_desc[READ],
+		      sock->recv_buffer + sock->recv_buffer_fill,
+		      do_read, (DWORD *) &num_read, sock->overlap[READ]))
+    {
+      if (GetLastError () != ERROR_IO_PENDING)
+        {
+	  svz_log (LOG_ERROR, "pipe: ReadFile: %s\n", SYS_ERROR);
+	  return -1;
+        }
+
+      /* Schedule the pipe for the GetOverlappedResult() call. */
+      sock->recv_pending = do_read;
+      sock->flags |= SOCK_FLAG_READING;
+      return 0;
     }
 #else /* not __MINGW32__ */
   if ((num_read = read (sock->pipe_desc[READ],
@@ -395,7 +500,7 @@ svz_pipe_read_socket (svz_socket_t *sock)
 #endif /* not __MINGW32__ */
 
   /* Some data has been read from the pipe. */
-  else if (num_read > 0)
+  if (num_read > 0)
     {
       sock->last_recv = time (NULL);
 
@@ -416,7 +521,7 @@ svz_pipe_read_socket (svz_socket_t *sock)
     }
 
 #ifndef __MINGW32__
-  /* the socket was selected but there is no data */
+  /* The pipe was selected but there is no data. */
   else
     {
       svz_log (LOG_ERROR, "pipe: read: no data on pipe %d\n", 
@@ -438,33 +543,20 @@ svz_pipe_write_socket (svz_socket_t *sock)
 {
   int num_written, do_write;
 
-  /* 
-   * Write as many bytes as possible, remember how many
-   * were actually sent. Do not write more than the content
-   * length of the post data.
-   */
+  /* Write as many bytes as possible, remember how many were actually 
+     sent. */
   do_write = sock->send_buffer_fill;
 
 #ifdef __MINGW32__
   /* Named pipes in Win32 cannot transfer more than 64KB at once. */
-  if (do_write > 64 * 1024)
-    do_write = 64 * 1024;
+  if (do_write > PIPE_MAX_WRITE)
+    do_write = PIPE_MAX_WRITE;
 
-  if (!WriteFile (sock->pipe_desc[WRITE], sock->send_buffer, 
-		  do_write, (DWORD *) &num_written, sock->overlap[WRITE]))
+  /* Data bytes have been stored in system's cache. Now we are checking 
+     if pending write operation has been completed. */
+  if (sock->flags & SOCK_FLAG_WRITING)
     {
-      if (GetLastError () != ERROR_IO_PENDING)
-	{
-	  svz_log (LOG_ERROR, "pipe: WriteFile: %s\n", SYS_ERROR);
-	  return -1;
-	}
-
-      /* 
-       * Data bytes have been stored in system's cache. Now we are
-       * checking if pending write operation has been completed.
-       */
-      if (!GetOverlappedResult (sock->pipe_desc[WRITE], 
-				sock->overlap[WRITE], 
+      if (!GetOverlappedResult (sock->pipe_desc[WRITE], sock->overlap[WRITE], 
 				(DWORD *) &num_written, FALSE))
 	{
 	  if (GetLastError () != ERROR_IO_INCOMPLETE)
@@ -473,8 +565,33 @@ svz_pipe_write_socket (svz_socket_t *sock)
 		       SYS_ERROR);
 	      return -1;
 	    }
+	  return 0;
 	}
-      num_written = do_write;
+
+      /* Reschedule the pipe descriptor for yet another WriteFile(). */
+      else
+	{
+	  sock->send_pending -= num_written;
+	  sock->flags &= ~SOCK_FLAG_WRITING;
+	  if (sock->send_pending != 0)
+	    {
+	      svz_log (LOG_ERROR, "pipe: %d pending send bytes left\n",
+		       sock->send_pending);
+	    }
+	}
+    }
+  /* Really write to the pipe. */
+  else if (!WriteFile (sock->pipe_desc[WRITE], sock->send_buffer, 
+		       do_write, (DWORD *) &num_written, sock->overlap[WRITE]))
+    {
+      if (GetLastError () != ERROR_IO_PENDING)
+	{
+	  svz_log (LOG_ERROR, "pipe: WriteFile: %s\n", SYS_ERROR);
+	  return -1;
+	}
+      sock->send_pending += do_write;
+      sock->flags |= SOCK_FLAG_WRITING;
+      return 0;
     }
 #else /* not __MINGW32__ */
   if ((num_written = write (sock->pipe_desc[WRITE], 
@@ -514,12 +631,13 @@ svz_pipe_create (HANDLE recv_fd, HANDLE send_fd)
 {
   svz_socket_t *sock;
 
-
+#ifndef __MINGW32__
   /* Try to set to non-blocking I/O. */
   if (svz_fd_nonblock ((int) recv_fd) != 0)
     return NULL;
   if (svz_fd_nonblock ((int) send_fd) != 0)
     return NULL;
+#endif /* __MINGW32__ */
 
   /* Do not inherit these pipes */
   if (svz_fd_cloexec ((int) recv_fd) != 0)
@@ -694,10 +812,20 @@ svz_pipe_set_files (svz_socket_t *sock, char *recv, char *send)
   sock->recv_pipe = svz_strdup (recv);
   sock->send_pipe = svz_strdup (send);
 #else /* __MINGW32__ */
-  sock->recv_pipe = svz_malloc (strlen (recv) + 10);
-  sprintf (sock->recv_pipe, "\\\\.\\pipe\\%s", recv);
-  sock->send_pipe = svz_malloc (strlen (send) + 10);
-  sprintf (sock->send_pipe, "\\\\.\\pipe\\%s", send);
+  if (strstr (recv, "\\pipe\\") == NULL)
+    {
+      sock->recv_pipe = svz_malloc (strlen (recv) + 10);
+      sprintf (sock->recv_pipe, "\\\\.\\pipe\\%s", recv);
+    }
+  else
+    sock->recv_pipe = svz_strdup (recv);
+  if (strstr (send, "\\pipe\\") == NULL)
+    {
+      sock->send_pipe = svz_malloc (strlen (send) + 10);
+      sprintf (sock->send_pipe, "\\\\.\\pipe\\%s", send);
+    }
+  else
+    sock->send_pipe = svz_strdup (send);
 #endif /* __MINGW32__ */
 }
 
@@ -965,7 +1093,7 @@ svz_pipe_listener (svz_socket_t *sock, svz_pipe_t *recv, svz_pipe_t *send)
   recv_pipe = CreateNamedPipe (
     sock->recv_pipe,                            /* path */
     PIPE_ACCESS_INBOUND | FILE_FLAG_OVERLAPPED, /* receive + overlapped */
-    PIPE_READMODE_BYTE | PIPE_NOWAIT,           /* binary + non-blocking */
+    PIPE_READMODE_BYTE | PIPE_WAIT,             /* binary + blocking */
     1,                                          /* one instance only */
     0, 0,                                       /* default buffer sizes */
     100,                                        /* timeout in ms */
@@ -982,7 +1110,7 @@ svz_pipe_listener (svz_socket_t *sock, svz_pipe_t *recv, svz_pipe_t *send)
   send_pipe = CreateNamedPipe (
     sock->send_pipe,                             /* path */
     PIPE_ACCESS_OUTBOUND | FILE_FLAG_OVERLAPPED, /* send + overlapped */
-    PIPE_TYPE_BYTE | PIPE_NOWAIT,                /* bin + non-blocking */
+    PIPE_TYPE_BYTE | PIPE_WAIT,                  /* binary + blocking */
     1,                                           /* one instance only */
     0, 0,                                        /* default buffer sizes */
     100,                                         /* timeout in ms */
