@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: server-socket.c,v 1.19 2001/09/11 15:05:48 ela Exp $
+ * $Id: server-socket.c,v 1.21 2001/11/29 23:41:43 raimi Exp $
  *
  */
 
@@ -170,12 +170,7 @@ svz_server_create (svz_portcfg_t *port)
    */
   if (port->proto & (PROTO_TCP | PROTO_PIPE))
     {
-      svz_free (sock->recv_buffer);
-      svz_free (sock->send_buffer);
-      sock->recv_buffer_size = 0;
-      sock->send_buffer_size = 0;
-      sock->recv_buffer = NULL;
-      sock->send_buffer = NULL;
+      svz_sock_resize_buffers (sock, 0, 0);
       sock->check_request = svz_sock_detect_proto; 
     }
 
@@ -205,18 +200,16 @@ svz_server_create (svz_portcfg_t *port)
 	}
       else if (port->proto & PROTO_UDP)
 	{
-	  svz_sock_resize_buffers (sock, port->send_buffer_size,
-				   port->recv_buffer_size);
-	  sock->read_socket = svz_udp_read_socket;
+	  svz_sock_resize_buffers (sock, 0, 0);
+	  sock->read_socket = svz_udp_lazy_read_socket;
 	  sock->write_socket = svz_udp_write_socket;
 	  sock->check_request = svz_udp_check_request;
 	  proto = "udp";
 	}
       else if (port->proto & PROTO_ICMP)
 	{
-	  svz_sock_resize_buffers (sock, port->send_buffer_size,
-				   port->recv_buffer_size);
-	  sock->read_socket = svz_icmp_read_socket;
+	  svz_sock_resize_buffers (sock, 0, 0);
+	  sock->read_socket = svz_icmp_lazy_read_socket;
 	  sock->write_socket = svz_icmp_write_socket;
 	  sock->check_request = svz_icmp_check_request;
 	  sock->itype = port->icmp_type;
@@ -249,7 +242,7 @@ svz_tcp_accept (svz_socket_t *server_sock)
   struct sockaddr_in client;	/* address of connecting clients */
   socklen_t client_size;	/* size of the address above */
   svz_socket_t *sock;
-  svz_portcfg_t *port = server_sock->port;;
+  svz_portcfg_t *port = server_sock->port;
 
   memset (&client, 0, sizeof (client));
   client_size = sizeof (client);
@@ -310,6 +303,7 @@ svz_tcp_accept (svz_socket_t *server_sock)
 			       port->recv_buffer_size);
       svz_sock_enqueue (sock);
       svz_sock_setparent (sock, server_sock);
+      sock->proto = server_sock->proto;
       svz_sock_connections++;
 
       /* Check access and connect frequency here. */
@@ -502,6 +496,7 @@ svz_pipe_accept (svz_socket_t *server_sock)
 			   port->recv_buffer_size);
   svz_sock_enqueue (sock);
   svz_sock_setparent (sock, server_sock);
+  sock->proto = server_sock->proto;
 
   svz_log (LOG_NOTICE, "%s: accepting client on pipe (%d-%d)\n",
 	   server_sock->recv_pipe, 
