@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: socket.c,v 1.16 2001/08/13 06:56:43 ela Exp $
+ * $Id: socket.c,v 1.20 2001/11/29 23:41:43 raimi Exp $
  *
  */
 
@@ -152,7 +152,9 @@ svz_sock_detect_proto (svz_socket_t *sock)
 	    return -1;
 	  if (server->connect_socket (server, sock))
 	    return -1;
-	  return sock->check_request (sock);
+	  if (sock->check_request)
+	    return sock->check_request (sock);
+	  return 0;
 	}
     }
 
@@ -363,6 +365,7 @@ svz_sock_alloc (void)
   sock->sock_desc = (SOCKET) -1;
   sock->pipe_desc[READ] = INVALID_HANDLE;
   sock->pipe_desc[WRITE] = INVALID_HANDLE;
+  sock->pid = INVALID_HANDLE;
 
   sock->read_socket = svz_tcp_read_socket;
   sock->write_socket = svz_tcp_write_socket;
@@ -387,7 +390,8 @@ svz_sock_alloc (void)
  * Resize the send and receive buffers for the socket @var{sock}. 
  * @var{send_buf_size} is the new size for the send buffer, 
  * @var{recv_buf_size} for the receive buffer. Note that data may be lost 
- * when the buffers shrink.
+ * when the buffers shrink. For a new buffer size of 0 the buffer is
+ * freed and the pointer set to NULL.
  */
 int 
 svz_sock_resize_buffers (svz_socket_t *sock, 
@@ -395,11 +399,22 @@ svz_sock_resize_buffers (svz_socket_t *sock,
 {
   char *send, *recv;
 
-  if (sock->send_buffer_size != send_buf_size)
+  if (send_buf_size == 0)
+    {
+      svz_free (sock->send_buffer);
+      send = NULL;
+    }
+  else if (sock->send_buffer_size != send_buf_size)
     send = svz_realloc (sock->send_buffer, send_buf_size);
   else
     send = sock->send_buffer;
-  if (sock->recv_buffer_size != recv_buf_size)
+
+  if (recv_buf_size == 0)
+    {
+      svz_free (sock->recv_buffer);
+      recv = NULL;
+    }
+  else if (sock->recv_buffer_size != recv_buf_size)
     recv = svz_realloc (sock->recv_buffer, recv_buf_size);
   else
     recv = sock->recv_buffer;
@@ -568,11 +583,14 @@ svz_sock_disconnect (svz_socket_t *sock)
   /* shutdown client connection */
   if (sock->flags & SOCK_FLAG_CONNECTED)
     {
-      if (shutdown (sock->sock_desc, 2) < 0)
-	svz_log (LOG_ERROR, "shutdown: %s\n", NET_ERROR);
+      if (!(sock->flags & SOCK_FLAG_NOSHUTDOWN))
+	{
+	  if (shutdown (sock->sock_desc, 2) < 0)
+	    svz_log (LOG_ERROR, "shutdown: %s\n", NET_ERROR);
+	}
       svz_sock_connections--;
     }
-      
+
   /* close the server/client socket */
   if (closesocket (sock->sock_desc) < 0)
     svz_log (LOG_ERROR, "close: %s\n", NET_ERROR);
