@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: udp-socket.c,v 1.13 2001/09/19 09:49:19 ela Exp $
+ * $Id: udp-socket.c,v 1.17 2001/11/29 23:41:43 raimi Exp $
  *
  */
 
@@ -55,6 +55,7 @@
 #include "libserveez/core.h"
 #include "libserveez/server-core.h"
 #include "libserveez/server.h"
+#include "libserveez/portcfg.h"
 #include "libserveez/udp-socket.h"
 
 /*
@@ -136,6 +137,23 @@ svz_udp_read_socket (svz_socket_t *sock)
   return 0;
 }
 
+/*
+ * This routine is the default reader for UDP server sockets. It allocates
+ * necessary buffers (that's why it's called lazy) and reverts to the default
+ * @code{svz_udp_read_socket()}.
+ */
+int
+svz_udp_lazy_read_socket (svz_socket_t *sock)
+{
+  svz_portcfg_t *port = sock->port;
+
+  svz_sock_resize_buffers (sock, port->send_buffer_size,
+			   port->recv_buffer_size);
+  sock->read_socket = svz_udp_read_socket;
+
+  return sock->read_socket (sock);
+}
+
 /*
  * The @code{svz_udp_write_socket()} callback should be called whenever 
  * the UDP socket descriptor is ready for sending. It sends a single packet 
@@ -240,7 +258,7 @@ svz_udp_check_request (svz_socket_t *sock)
   svz_array_foreach (sock->data, server, n)
     {
       sock->cfg = server->cfg;
-      
+
       if (server->handle_request)
 	{
 	  if (!server->handle_request (sock, sock->recv_buffer,
@@ -362,11 +380,11 @@ svz_udp_connect (unsigned long host, unsigned short port)
 
   /* Create a client socket. */
   if ((sockfd = svz_socket_create (PROTO_UDP)) == (SOCKET) -1)
-      return NULL;
+    return NULL;
 
   /* Try to connect to the server. Does it make sense for ICMP ? */
   if (svz_socket_connect (sockfd, host, port) == -1)
-     return NULL;
+    return NULL;
 
   /* Create socket structure and enqueue it. */
   if ((sock = svz_sock_alloc ()) == NULL)
@@ -378,6 +396,7 @@ svz_udp_connect (unsigned long host, unsigned short port)
   svz_sock_resize_buffers (sock, UDP_BUF_SIZE, UDP_BUF_SIZE);
   svz_sock_unique_id (sock);
   sock->sock_desc = sockfd;
+  sock->proto = PROTO_UDP;
   sock->flags |= (SOCK_FLAG_SOCK | SOCK_FLAG_CONNECTED | SOCK_FLAG_FIXED);
   svz_sock_enqueue (sock);
   svz_sock_intern_connection_info (sock);
